"""Cataloging and searching components for access codes.
"""
import grok
from hurry.query import Eq, Text
from hurry.query.query import Query
from zope.index.text.parsetree import ParseError
from waeup.sirp.interfaces import IUniversity, IQueryResultItem
from waeup.sirp.accesscodes.interfaces import IAccessCode

class AccessCodeIndexes(grok.Indexes):
    """A catalog for access codes.
    """
    grok.site(IUniversity)
    grok.name('accesscodes_catalog')
    grok.context(IAccessCode)

    code = grok.index.Field(attribute='representation')
    history = grok.index.Text(attribute='history')
    disabled = grok.index.Field(attribute='disabled')
    used = grok.index.Field(attribute='used')
    batch_serial = grok.index.Field(attribute='batch_serial')
    status = grok.index.Field(attribute='status')

class AccessCodeQueryResultItem(object):
    grok.implements(IQueryResultItem)

    def __init__(self, context, view):
        self.context = context
        #self.url = view.url(context)
        self.code = context.representation
        self.history = context.history
        self.status = context.status
        self.batch_serial = context.batch_serial

def search(query=None, searchtype=None, view=None):
    if not query:
        view.flash('Empty search string.')
        return
    hitlist = []
    try:
        if searchtype == 'history':
            results = Query().searchResults(
                Text(('accesscodes_catalog', searchtype), query))
        elif searchtype == 'batch_serial':
            results = Query().searchResults(
                Eq(('accesscodes_catalog', searchtype), int(query)))
        else:
            results = Query().searchResults(
                Eq(('accesscodes_catalog', searchtype), query))
        for result in results:
            hitlist.append(AccessCodeQueryResultItem(result, view=view))
    except ParseError:
        view.flash('Search string not allowed.')
        return
    return hitlist


